#json to read json file
import os
#import tkFileDialog

try:    
    import json
except ImportError:
    raise ImportError('<Please Install Python json Library! Then Run This Code Again.>')
try:
    import sys
except ImportError:
    raise ImportError('<Please Install Python sys Library! Then Run This Code Again.>')

# psycopg2 to open connection with PostgreSQL database 
try:    
    import psycopg2
    from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT # <-- ADD THIS LINE
except ImportError:
    raise ImportError('<Please Install Python psycopg2 Library! Then Run This Code Again.>')
#For GUI
try:    
    import Tkinter
    from Tkinter import *
except ImportError:
    raise ImportError('<Please Install Python Tkinter Library! Then Run This Code Again.>')

try:    
    import ttk
except ImportError:
    raise ImportError('<Please Install Python ttk Library! Then Run This Code Again.>')
try:    
    import tkFileDialog as fdlg
except ImportError:
    raise ImportError('<Please Install Python tkFileDialog Library! Then Run This Code Again.>')

import tkMessageBox
class FileSelDlgDemo(ttk.Frame):
    GPSFilePath=""
    Road_FilePath=""
    def __init__(self, isapp=True, name='fileseldlgdemo'):
        ttk.Frame.__init__(self, name=name)
        self.pack(expand="y", fill="both")
        self.master.title('TrajAnalytics Create Database')
        self._create_demo_panel()
                
    def _create_demo_panel(self):
        demoPanel = Frame(self)
        demoPanel.pack(side="top", fill="both", expand="y")
        
        
        frame = ttk.Frame(demoPanel)
        lbl = ttk.Label(frame, width=45,
                            text='Select a CSV file of the GPS trajectory data to {} '.format("open"))
        ent = ttk.Entry(frame, width=35)
        btn = ttk.Button(frame, text='Browse...', 
                             command=lambda i="open", e=ent: self._file_dialog(i, e,".CSV",'CSV files'))
        lbl.pack(side="left")
        ent.pack(side="left", expand="y", fill="x")
        btn.pack(side="left", padx=5)
        frame.pack(fill="x", padx='1c', pady=3)
        ####
        frame = ttk.Frame(demoPanel)
        lbl = ttk.Label(frame, width=45,
                            text='Select a JSON file of the road network data to {} '.format("open"))
        ent = ttk.Entry(frame, width=35)
        btn = ttk.Button(frame, text='Browse...', 
                             command=lambda i="open", e=ent: self._file_dialog(i, e,".JSON",'JSON files'))
        lbl.pack(side="left")
        ent.pack(side="left", expand="y", fill="x")
        btn.pack(side="left", padx=5)
        frame.pack(fill="x", padx='1c', pady=3)
        #####
        frame = ttk.Frame(demoPanel)        
        btn = ttk.Button(frame, text='Click to Create TrajAnalytics Database...', 
                             command=lambda i="create":self._createDB(i))        
        
        btn.pack(side="left", padx=150)
        frame.pack(fill="x", padx='1c', pady=3)
        
               
    def _file_dialog(self, type, ent,exct,ftype):
        # triggered when the user clicks a 'Browse' button 
        fn = None
        opts = {'initialfile': ent.get(),
                'filetypes': ((ftype, exct),)}
            
        opts['title'] = 'Select a file to open...'
        fn = fdlg.askopenfilename(**opts)
        global GPSFilePath,Road_FilePath
        if exct==".CSV":
            self.GPSFilePath=fn
        else:
            self.Road_FilePath=fn
            
        if fn:
            ent.delete(0, END)
            ent.insert(END, fn)
    

    def popup_showinfo(self,m):
        tkMessageBox.showinfo("TrajAnalytics", m)
    def _createDB(self,f):
   
        # config.json contains database configration information.
        with open('config.json') as info_file:
            conn_info = json.load(info_file)
        host=conn_info['host']
        dbname=conn_info['dbname']
        user=conn_info['user']
        password=conn_info['password']
        TrajUserName=conn_info['TrajUserName']
        Trajdbname=conn_info['Trajdbname']
        
        conn_string = "host='"+host+"' dbname='"+dbname+"' user='"+user+"' password='"+password+"'"
        print ("The following line shows the database connection information as what you have on the config.json file. ")
        print("")
        print (conn_string)
        print("")
        try:
            conn = psycopg2.connect(conn_string)
            conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT) # <-- ADD THIS LINE
            cursor = conn.cursor()
            print ("The connection with the PostgreSql Database has been successfully started.")
            print("")                
        except:
            print ("Please check the following! then run this code again ")
            print ("1. You have installed the PostgreSQL database and PostGIS extension correctly.")
            print ("2. The connection information in the config.json file are correct.")
            sys.exit()           
        db_name1= "traja_"+Trajdbname+"_1_1_"+TrajUserName;
        cursor.execute("DROP DATABASE IF EXISTS "+db_name1)
        cursor.execute("CREATE DATABASE "+db_name1)
        conn.commit()
        conn.close()
        conn_string1 = "host='"+host+"' dbname='"+db_name1+"' user='"+user+"' password='"+password+"'"    
        conn = psycopg2.connect(conn_string1)
        cursor = conn.cursor()
        cursor.execute("CREATE EXTENSION postgis")# add PostGis Extension
        cursor.execute("CREATE EXTENSION btree_gin")            
        cursor.execute("CREATE TABLE gps_points(tripid int,pdatetime timestamp without time zone,latitude double precision,\
        longitude double precision,speed double precision,rid integer,highway character varying)");
        conn.commit()
        try:
            cursor.execute("COPY gps_points FROM '"+self.GPSFilePath+"' DELIMITER ',' CSV HEADER");            
        except:
            print ("Sorry The format of your file is not correct! ")
        cursor.execute("Drop TABLE IF EXISTS gps_points1 "); 
        cursor.execute("Create table gps_points1 as select tripid,ST_SetSRID(ST_MakePoint(longitude,latitude),4326) as point,\
        pdatetime,speed from gps_points");
        
        cursor.execute("Create INDEX gps_points1_Index ON gps_points1 USING gist (point)");
        cursor.execute("Drop TABLE IF EXISTS gps_points_rn "); 
        cursor.execute("Create table gps_points_rn as select tripid,ST_SetSRID(ST_MakePoint(longitude,latitude),4326) as point,\
        pdatetime,speed,rid,highway from gps_points");
        cursor.execute("Create INDEX gps_points_rn_Index ON gps_points_rn USING gist (point)");        
        cursor.execute("drop table gps_points")    
        conn.commit()
        
        cursor.execute("Drop TABLE IF EXISTS td"); 
        cursor.execute("create table td as (select *,ST_SetSRID(ST_StartPoint(trip),4326) as startpoint,\
        ST_SetSRID(ST_EndPoint(trip),4326)as endpoint from(select tripid,ST_SetSRID(ST_MakeLine(Point order by PDateTime),4326) as trip\
        ,min(PDateTime) as starttime,EXTRACT(DOW FROM min(PDateTime)) as startday,EXTRACT(HOUR FROM min(PDateTime)) as starthour,\
        max(PDateTime) as endtime,Avg(Speed) as avspeed ,min(Speed) as minspeed,max(Speed) as maxspeed,array_agg(Speed order by PDateTime) as speeds,\
        array_agg(PDateTime order by PDateTime) as pointstime FROM gps_points1 GROUP BY tripid)as x)")
        cursor.execute("CREATE UNIQUE INDEX td_indx0 ON td USING btree(tripid)")
        cursor.execute("CREATE INDEX td_indx2 ON td USING gist(trip)")
        cursor.execute("CREATE INDEX td_indx1 ON td USING GIN (tripid,startday,starthour,starttime,speeds,avspeed,\
        pointstime)")
        cursor.execute("Drop TABLE IF EXISTS tds"); 
        cursor.execute("create table tds as (select *,ST_SetSRID(ST_StartPoint(trip),4326) as startpoint,\
        ST_SetSRID(ST_EndPoint(trip),4326)as endpoint from(select tripid,ST_SetSRID(ST_MakeLine(Point order by PDateTime),4326) as trip\
        ,min(PDateTime) as starttime,EXTRACT(DOW FROM min(PDateTime)) as startday,EXTRACT(HOUR FROM min(PDateTime)) as starthour,\
        max(PDateTime) as endtime,Avg(Speed) as avspeed ,min(Speed) as minspeed,max(Speed) as maxspeed,array_agg(Speed order by PDateTime) as speeds,\
        array_agg(rid order by PDateTime) as orids,array_agg(highway order by PDateTime) as roads_type,array_agg(PDateTime order by PDateTime) as\
        pointstime FROM gps_points_rn GROUP BY tripid)as x)")
        cursor.execute("CREATE UNIQUE INDEX tds_indx0 ON tds USING btree(tripid)")
        cursor.execute("CREATE INDEX tds_indx2 ON tds USING gist(trip)")
        cursor.execute("CREATE INDEX tds_indx1 ON tds USING GIN (tripid,startday,starthour,starttime,speeds,avspeed,\
        pointstime,orids,roads_type)")
        conn.commit()
        # import road data
        with open(self.Road_FilePath) as json_file:  
            geojson = json.load(json_file)
        
        x=json.dumps(geojson)
        cursor.execute("Drop TABLE IF EXISTS osm_roads_car"); 
        sqlrn="create table osm_roads_car as (WITH data AS (Select '"+x+"'::json AS fc) Select f2.geom,(f2.properties->>'osm_id')::integer as osm_id,\
        (f2.properties->>'highway')::varchar as highway,f2.properties->>'lanes' as lanes,f2.properties->>'oneway' as oneway from \
        (SELECT  ST_SetSRID(ST_GeomFromGeoJSON(feat->>'geometry'),4326) as geom,feat->'properties' AS properties FROM ( \
        SELECT json_array_elements(fc->'features')\
        AS feat FROM data ) AS f)as f2)"
        cursor.execute(sqlrn)    
        cursor.execute("CREATE INDEX osm_roads_car_indx ON osm_roads_car USING gist(geom)")
        cursor.execute("CREATE INDEX osm_roads_car_indx0 ON osm_roads_car USING btree (osm_id,highway)")    
        conn.commit()
        M="The TrajAnalytics ( " +str(db_name1)+" ) database has been successfully created."
        print(M)
        self.popup_showinfo(M)
if __name__ == '__main__':
    FileSelDlgDemo().mainloop()


    


